 {{   EDDIE Wireless Controller    V.1   Nicholas G. Lordi   2/2012    Eddie_B_Control.spin

     This software is designed to access the Eddie robotic platform via XBee wireless connections.
     The firmware loaded on the Eddie controller board is Eddie_B.spin.  The portable controller
     includes a 16 key membrane keypad, a serial 2x16 LCD display, a XBee PRO, an
     optional SD card socket, and a rechargeable RC power supply connected to a Parallax Proto
     Board.

     The user can select among 4 modes of operation:  A - Position, B - Velocity, C - Power, and
     D - Read/Get Data.  Each mode includes upto 10 possible commands.  The user first selects
     the desired mode (A, B, C, or D), then a number 0 - 9. The device transmits the command
     corresponding to the number to the Eddie, which responds to the command.  Pressing * exits the
     selected mode, allowing selection of a different mode.  Pressing #  stops Eddie.  This key is
     active, irrespective of the selected mode.  Following is a list of the allowed commands.

     Note that the LCD will display the selected mode and command on the first line.  The second line
     is used to display data transmitted to the controller from Eddie, eg. a parameter value. Key
     presses are used to increment/decrement critical parameter values rather than entering numbers.
     The resulting values are echoed back to the controller display.

     KEY   A (Position)          B (Velocity)               C (Power)              D (Read Data)   

      1    Go Forward            Increase Lft Motor Vel     Increase Lft Power     Get Heading
      2    Go Backward           Decrease Lft Motor Vel     Decrease Lft Power     Get Lft Distance
      3    Increase Distance     Increase Rgt Motor Vel     Increase Rgt Power     Get Rgt Distance
      4    Decrease Distance     Decrease Rgt Motor Vel     Decrease Rgt Power     Get Left Speed
      5    Turn Left             Set Speed Lft+ Rgt+        Set Power Lft+ Rgt+    Get Right Speed
      6    Turn Right            Set Speed Lft- Rgt+        Set Power Lft- Rgt+    Read Left Ping                                                                                            
      7    Increase Degrees      Set Speed Lft+ Rgt-        Set Power Lft+ Rgt-    Read Right Ping
      8    Decrease Degrees      Set Speed Lft- Rgt-        Set Power Lft- Rgt-    Read Left ADC
      9                          Increase Acceleration                             Read Center ADC
      0                          Decrease Acceleration                             Read Right ADC
      A                          Set Acceleration
      #    Stop                  Stop                       Stop                   Reset

      Units: Velocity - positions/sec   Distance - positions        186 positions per wheel rotation



}}




CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
          
  XB_Tx   = 1
  XB_Rx   = 0
  XB_Baud = 9600

  LCD_Rx  = 7
  LCD_Baud = 19200

OBJ
  
  kp   : "4x4 Keypad Reader2"
  xb   : "Xbee_Object"
  lcd  : "LCD_DisplayDriver"

VAR

  long  Value
  word  key, set
  

PUB start

  
  xb.start(XB_Rx, XB_Tx, 0, XB_Baud)
  lcd.Init(LCD_Rx, LCD_Baud, false, 2)
   


  repeat

    set := kp.ReadKeyPad
    lcd.ClearDisplay
    CASE set                         
      $1     :  lcd.OutStr(string("A "))
                SetA                
      $10    :  lcd.OutStr(string("B "))
                SetB
      $100   :  lcd.OutStr(string("C "))
                SetC
      $1000  :  lcd.OutStr(string("D "))
                SetD               
      


PRI  SetA

  set := $0  
  if key == $8000
     key := $0
  repeat until key == $8000
     lcd.GoToRowCol(0,2)
     key := kp.ReadKeyPad     '<-- One line command to read the 4x4 keypad
      
     CASE key
          $8    : xb.str(string("TRVL 1"))       ' key = 1 Go Forward
                  xb.CR
                  lcd.OutStr(@Fwrd)
                                                                         
          $4    : xb.str(string("TRVL 0"))        ' key = 2 Go Backward                                                
                  xb.CR
                  lcd.OutStr(@Bwrd)
                  
          $2    : xb.str(string("DIST 0"))             ' key = 3 Increment Distance (+50)
                  xb.CR
                  lcd.OutStr(string("Incr Distance"))
                  getData
                        
          $80   : xb.str(string("DIST 1"))         ' key = 4 Decrement Distance (-50)
                  xb.CR 
                  lcd.OutStr(string("Decr Distance"))
                  getData
                      
          $40   : xb.str(string("TURN 1"))  ' key = 5 Turn Right
                  xb.CR
                  lcd.OutStr(@GoRgt)
                      
          $20   : xb.str(string("TURN 0"))   ' key = 6 Turn Left
                  xb.CR
                  lcd.OutStr(@GoLft)
                              
          $800  : xb.str(string("DEG 0"))      ' key = 7 Increment Degrees (+5)
                  xb.CR
                  lcd.OutStr(string("Incr Degrees"))
                  getData
                                                    
          $400  : xb.str(string("DEG 1"))      ' key = 8 Decrement Degrees (-5) 
                  xb.CR
                  lcd.OutStr(string("Decr Degrees"))
                  getData          
          
          $2000 : xb.str(string("STOP 0A"))  ' key = # STOP Motors
                  xb.CR
                  lcd.OutStr(string("STOP      "))   

                
     Pause(300)   

PRI SetB

  set := $0  
  if key == $8000
     key := $0
  repeat until key == $8000
     lcd.GoToRowCol(0,2)
     key := kp.ReadKeyPad     '<-- One line command to read the 4x4 keypad
      
     CASE key

          $8    : xb.str(string("LSPD 0"))     ' key = 1 Increment Left Motor Velocity
                  xb.CR
                  lcd.OutStr(string("Incr Lft Vel"))
                  getData    
     
          $4    : xb.str(string("LSPD 1"))      ' key = 2 Decrement Left Motor Velocity
                  xb.CR
                  lcd.OutStr(string("Decr Lft Vel"))
                  getData 
     
          $2    :  xb.str(string("RSPD 0"))       ' key = 3 Increment Right Motor Velocity
                   xb.CR         
                   lcd.OutStr(string("Incr Rgt Vel"))
                   getData 
          
          $80   :  xb.str(string("RSPD 1"))       ' key = 4 Decrement Right Motor Velocity
                   xb.CR
                   lcd.OutStr(string("Decr Rgt Vel"))
                   getData  
                  
          $40   :  xb.str(string("GOSPD 0"))       ' key = 5 Set Left(+) & Right(+) Motor Speeds
                   xb.CR
                   lcd.OutStr(string("Set Speed ++"))

          $20   :  xb.str(string("GOSPD 1"))       ' key = 6  Set Left(-) & Right(+) Motor Speeds 
                   xb.CR
                   lcd.OutStr(string("Set Speed -+"))
                      
          $800  :  xb.str(string("GOSPD 2"))       ' key = 7  Set Left(+) & Right(-) Motor Speeds
                   xb.CR
                   lcd.OutStr(string("Set Speed +-")) 
                      
          $400  :  xb.str(string("GOSPD 3"))       ' key = 8  Set Left(-) and Right(-) Motor Speeds
                   xb.CR
                   lcd.OutStr(string("Set Speed --"))
                              
          $200  :  xb.str(string("ACEL 0"))        ' key = 9 Increment Acceleration
                   xb.CR
                   lcd.OutStr(string("Acceleration +"))                        

          $4000 :  xb.str(string("ACEL 1"))        ' key = 0  Decrement Acceleration
                   xb.CR
                   lcd.OutStr(string("Acceleration -"))  
                          
          $1    :  xb.str(string("ACC"))           ' key = A  Set Wheel Accelerations
                   xb.Cr
                   lcd.OutStr(string("Set Acceler."))          
                                                                      
          $2000 :  xb.str(string("STOP 0A"))       ' key = # STOP Motors
                   xb.CR
                   lcd.OutStr(string("STOP      ")) 
                
     Pause(300)   

PRI SetC

  set := $0  
  if key == $8000
     key := $0
  repeat until key == $8000
     lcd.GoToRowCol(0,2)
     key := kp.ReadKeyPad     '<-- One line command to read the 4x4 keypad
      
     CASE key
     
       $8      : xb.str(string("LPOW 0"))     ' key = 1 Increment Left Motor Power
                 xb.CR
                 lcd.OutStr(string("Incr Lft Power"))
                 getData 
          
       $4     : xb.str(string("LPOW 1"))       ' key = 2 Decrement Left Motor Power
                xb.CR
                lcd.OutStr(string("Decr Lft Power"))
                getData
                   
       $2     : xb.str(string("RPOW 0"))       ' key = 3 Increment Right Motor Power
                xb.CR
                lcd.OutStr(string("Incr Rgt Power"))
                getData 
          
       $80    : xb.str(string("RPOW 1"))       ' key = 4 Decrement Right Motor Power
                xb.CR
                lcd.OutStr(string("Decr Rgt Power"))
                getData

       $40    : xb.str(string("GO 0"))         ' key = 5  Set Left(+) & Right(+) Motor Powers
                xb.CR
                lcd.OutStr(string("Set Power ++"))

       $20    : xb.str(string("GO 1"))         ' key = 6  Set Left(-) & Right(+) Motor Powers 
                xb.CR
                lcd.OutStr(string("Set Power -+"))
                      
       $800   : xb.str(string("GO 2"))         ' key = 7  Set Left(+) & Right(-) Motor Powers
                xb.CR
                lcd.OutStr(string("Set Power +-")) 
                      
       $400   : xb.str(string("GO 3"))         ' key = 8  Set Left(-) and Right(-) Motor Powers
                xb.CR
                lcd.OutStr(string("Set Power --"))                        
      
                                                                     
       $2000  : xb.str(string("STOP 0A"))       ' key = # STOP Motors
                xb.CR
                lcd.OutStr(string("STOP Motors"))

     Pause(300)                                    


PRI SetD

  set := $0  
  if key == $8000
     key := $0
  repeat until key == $8000
     lcd.GoToRowCol(0,2)
     key := kp.ReadKeyPad     '<-- One line command to read the 4x4 keypad
      
     CASE key
     
      
                   
          $8   : xb.str(string("HEAD"))         ' key = 1  Get current heading relative to start.
                 xb.CR
                 lcd.OutStr(string("Heading"))
                 getData
                                         
          $4   : xb.str(string("DIST 0"))         ' key = 2  Get current distance left wheel has traveled.
                 xb.CR
                 lcd.OutStr(string("Lft Distance"))
                 getData

          $2   : xb.str(string("DIST 1"))         ' key = 3  Get current distance right wheel has traveled.
                 xb.CR
                 lcd.OutStr(string("Rgt Distance"))
                 getData        

          $80  :  xb.str(string("SPD 0"))        ' key = 4  Get current left wheel average speed. 
                  xb.CR
                  lcd.OutStr(string("Avg Spd Left"))
                  getData                    
                                     
          $40  :  xb.str(string("SPD 1"))       ' key = 5  Get current right wheel average velocity.
                  xb.CR
                  lcd.OutStr(string("Avg Spd Right"))
                  getData

          $20  :  xb.str(string("PING 0"))        ' key = 6  Read left Ping Value.
                  xb.CR
                  lcd.OutStr(string("Left Ping Value"))                  
                  getData
          $800 :  xb.str(string("PING 1"))        ' key = 7 Read Right Ping Value.
                  xb.CR
                  lcd.OutStr(string("Right Ping Valus"))
                  getData
                  

          $400 :  xb.str(string("ADC 0"))        ' key = 8  Read left ADC Value.
                  xb.CR
                  lcd.OutStr(string("Left ADC Value"))
                  getData

          $200 :  xb.str(string("ADC 1"))        ' key = 9  Read center ADC Value.
                  xb.CR
                  lcd.OutStr(string("Center ADC Value"))
                  getData

          $4000 : xb.str(string("ADC 2"))        ' key = 0  Read right ADC Value.
                  xb.CR
                  lcd.OutStr(string("Right ADC Value"))
                  getData 
                  

          $2000 : xb.str(string("RST"))         ' key = # Reset distances & heading to 0.
                  xb.CR
                  lcd.OutStr(string("Reset Distance"))                
                   
     Pause(300)

PRI getData

  Value := xb.rxDecTime(100)
  lcd.GoToRowCol(1,0)
  lcd.OutText(string("                 "))
  lcd.GoToRowCol(1,0)
  lcd.OutNum(Value)


  
PRI Pause(ms)

    waitcnt(cnt + ((clkfreq/1000) * ms))

DAT

  Fwrd   byte  "FORWARD   ",0
  Bwrd   byte  "BACKWARD  ",0
  GoLft  byte  "TURN LEFT ",0
  GoRgt  byte  "TURN RIGHT ",0


                                  